const system = require("../../../system");
const ServiceBase = require("../../sve.base");
const settings = require("../../../../config/settings");
const { PDICT, MsgTemplate } = require("../../../../config/platform");
class NeedSolutionService extends ServiceBase {
  constructor() {
    super("need", ServiceBase.getDaoName(NeedSolutionService));
    this.needInfoDao = system.getObject("db.need.needInfoDao");
    this.utilsMsgSendSve = system.getObject("service.utilsSve.utilsMsgSendSve");
  }

  /**
   * 发送消息通知
   * @param {*} needInfo 需求信息
   * @param {*} status 状态
   */
  async sendNotification(needInfo, status) {
    try {
      if(!needInfo || !status || !needInfo.consult_type){
        return system.getResultFail();
      }
      var nameArr = needInfo && needInfo.consult_type_name ? needInfo.consult_type_name.split("/") : [];
      var productName = null;
      if (nameArr && nameArr.length == 4) {
        productName = nameArr[2];
      }
      var typeOne =null;
      if (needInfo.consult_type.indexOf("/ic/") >= 0) {//工商产品
        typeOne = "ic";
      }
      if (needInfo.consult_type.indexOf("/qcfw/") >= 0) {//资质证照产品
        typeOne="qcfw";
      }
      if(!typeOne || !productName){
        return system.getResultFail();
      }
      var params = {};
      if(needInfo.contacts_mobile){//短信
        var sms = MsgTemplate[typeOne].need[status] && MsgTemplate[typeOne].need[status].sms ? MsgTemplate[typeOne].need[status].sms : "";
        if(sms){
          if(productName){
            sms = sms.replace(/{productName}/g, productName);
          }
          if(needInfo.region_name){
            sms = sms.replace(/{area}/g, needInfo.region_name);
          }
          if(needInfo.user_id){
            sms = sms.replace(/{userId}/g, needInfo.user_id);
          }
          if(needInfo.user_name){
            sms = sms.replace(/{userName}/g, needInfo.user_name);
          }
          if (sms.indexOf("{")<0 && sms.indexOf("}")<0) {
            params["phoneList"] = [{phoneNumber:needInfo.contacts_mobile,messageBody:sms}];
          }
        }
      }
      if(needInfo.user_id){//站内信
        var webinfo = MsgTemplate[typeOne].need[status] && MsgTemplate[typeOne].need[status].webinfo ? MsgTemplate[typeOne].need[status].webinfo : "";
        var webinfotitle = MsgTemplate[typeOne].need[status] && MsgTemplate[typeOne].need[status].webinfotitle ? MsgTemplate[typeOne].need[status].webinfotitle : "";
        if(webinfo && webinfotitle){
          if(productName){
            webinfo = webinfo.replace(/{productName}/g, productName);
          }
          if(needInfo.region_name){
            webinfo = webinfo.replace(/{area}/g, needInfo.region_name);
          }
          if(needInfo.user_id){
            webinfo = webinfo.replace(/{userId}/g, needInfo.user_id);
          }
          if(needInfo.user_name){
            webinfo = webinfo.replace(/{userName}/g, needInfo.user_name);
          }
          if (webinfo.indexOf("{")<0 && webinfo.indexOf("}")<0) {
            params["subAccountList"] = [{title:webinfotitle,subAccount:needInfo.user_id,messageBody:webinfo}];
          }
        }
        
      }
      if(needInfo.wechat_no){//微信
        var wechatMsg = MsgTemplate[typeOne].need[status] && MsgTemplate[typeOne].need[status].wx ? MsgTemplate[typeOne].need[status].wx : "";
        var wechattitle = MsgTemplate[typeOne].need[status] && MsgTemplate[typeOne].need[status].webinfotitle ? MsgTemplate[typeOne].need[status].webinfotitle : "";
        if(wechatMsg && wechattitle){
          if(productName){
            wechatMsg = wechatMsg.replace(/{productName}/g, productName);
          }
          if(needInfo.region_name){
            wechatMsg = wechatMsg.replace(/{area}/g, needInfo.region_name);
          }
          if(needInfo.solutionNum){
            wechatMsg = wechatMsg.replace(/{solutionNum}/g, needInfo.solutionNum);
          }
          if(needInfo.user_id){
            wechatMsg = wechatMsg.replace(/{userId}/g, needInfo.user_id);
          }
          if(needInfo.user_name){
            wechatMsg = wechatMsg.replace(/{userName}/g, needInfo.user_name);
          }
          if (wechatMsg.indexOf("{")<0 && wechatMsg.indexOf("}")<0) {
            params["wechatList"] = [{title:wechattitle,wechatNumber:needInfo.wechat_no,messageBody:wechatMsg}];
          }
        }
        
      }
      if(needInfo.email){//邮箱
        var emailMsg = MsgTemplate[typeOne].need[status] && MsgTemplate[typeOne].need[status].email ? MsgTemplate[typeOne].need[status].email : "";
        if(emailMsg){
          if(productName){
            emailMsg = emailMsg.replace(/{productName}/g, productName);
          }
          if(needInfo.region_name){
            emailMsg = emailMsg.replace(/{area}/g, needInfo.region_name);
          }
          if(needInfo.user_id){
            emailMsg = emailMsg.replace(/{userId}/g, needInfo.user_id);
          }
          if(needInfo.user_name){
            emailMsg = emailMsg.replace(/{userName}/g, needInfo.user_name);
          }
          if (emailMsg.indexOf("{")<0 && emailMsg.indexOf("}")<0) {
            params["emailList"] = [{email:needInfo.email,messageBody:emailMsg}];
          }
        }
      }
      await this.utilsMsgSendSve.sendMessageVerify(params);
      return system.getResultSuccess();
    } catch (e) {
      this.execClient.execLogs("needSolutionSve.js/sendNotification(发送短信通知)方法出现异常", { needInfo:needInfo, status: status }, "needSolutionSve_sendNotification_error", null, e.stack);
      return system.getResultFail();
    }
  }
  //服务商提交/修改方案
  async submitSolution(pobj) {
    if (!pobj || !pobj.actionBody) {
      return system.getResultFail(-100, "参数错误");
    }
    var ab = pobj.actionBody;
    if (!ab.needNum) {
      return system.getResultFail(-101, "需求号不能为空");
    }
    if (!ab.solutionContent) {
      return system.getResultFail(-102, "方案内容不能为空");
    }
    if (!ab.solutionContent.businessType) {
      return system.getResultFail(-103, "方案内容有误,商机类型不能为空");
    }

    var needinfo = await this.needInfoDao.model.findOne({
      where: { need_num: ab.needNum },
      raw: true
    });
    if (!needinfo || !needinfo.id) {
      return system.getResultFail(-300, "未知需求信息");
    }
    if (needinfo.status == 4 || needinfo.status == 5) {
      return system.getResultFail(-304, "操作失败，需求状态错误");
    }
    if (needinfo.consult_type != ab.solutionContent.businessType) {
      return system.getResultFail(-305, "商机类型与需求信息不符");
    }
    var needsolution = null;
    if (ab.solutionNum) {//方案编码，修改方案时传入，不传默认新增
      needsolution = await this.dao.model.findOne({
        where: { need_num: ab.needNum, solution_num: ab.solutionNum },
        raw: true
      });
      if (!needsolution || !needsolution.id) {
        return system.getResultFail(-301, "方案修改失败，未知方案信息");
      }
      if (needsolution.is_refusal != 1) {
        return system.getResultFail(-303, "只能修改用户驳回方案");
      }
      // var new_solution_num = await this.getBusUid("NS_");
      var updateObj = { id: needsolution.id, is_refusal: 0, solution_content: ab.solutionContent };
      var self = this;
      return await self.db.transaction(async function (t) {
        await self.dao.update(updateObj, t);
        await self.needInfoDao.update({ id: needinfo.id, status: "3" }, t);
        //发送短信通知
        needinfo.solutionNum = ab.solutionNum;
        self.sendNotification(needinfo, 3);
        return system.getResultSuccess();
      });

    } else {
      needsolution = await this.dao.model.findOne({
        where: { need_num: ab.needNum },
        raw: true
      });
      if (needsolution && needsolution.id) {
        return system.getResultFail(-302, "新增失败，已存在方案信息");
      }
      var solution_num = await this.getBusUid("NS_");
      var createObj = {
        solution_num: solution_num,
        need_num: ab.needNum,
        user_id: needinfo.user_id,
        user_name: needinfo.user_name,
        solution_content: ab.solutionContent
      };
      var self = this;
      return await self.db.transaction(async function (t) {
        await self.needInfoDao.update({ id: needinfo.id, status: "3" }, t);
        await self.dao.create(createObj, t);
        //发送消息通知
        needinfo.solutionNum = solution_num;
        self.sendNotification(needinfo, 3);
        return system.getResultSuccess(solution_num);
      });

    }
  }

  //驳回方案
  async refusalSolution(actionBody, req) {
    if (!actionBody.NeedNum) {
      return system.getResultFail(-102, "needNum cannot be empty");
    }
    if (!actionBody.SolutionNum) {
      return system.getResultFail(-102, "SolutionNum cannot be empty");
    }
    var self = this;
    return await self.db.transaction(async function (t) {
      var sobj = {
        need_num: actionBody.NeedNum
      }
      var needInfo = await self.needInfoDao.findOne(sobj, t);
      if (!needInfo) {
        return system.getResultFail(-102, "needInfo is empty");
      }
      sobj.solution_num = actionBody.SolutionNum;
      var solutionInfo = await self.dao.findOne(sobj, t);
      if (!solutionInfo) {
        return system.getResultFail(-102, "solutionInfo is empty");
      }
      var urlobj = {
        path_code: needInfo.dataValues.consult_type,
        region_id: needInfo.dataValues.region_id
      }
      var serviceinfo = await self.needInfoDao.findServiceByProductType(urlobj);
      if (serviceinfo.status < 0) {
        return system.getResultFail(-203, "servicer is empty");
      }
      //生产者------方案反馈
      var pushobj = {
        "actionType": "produceData",// Y 功能名称
        "actionBody": {
          notifyUrl: "",
          actionType: "needClose",
          pushUrl: serviceinfo.data.push_domain_addr + "/entService/consultation/springBoard",
          messageBody: actionBody,
          identifyCode: "needClose"
        }
      };
      var r = await self.execPostByTimeOut(req, pushobj, settings.opPushUrl());
      if (r.status < 1) {
        return system.getResultFail(-102, "Post error");
      }
      await self.needInfoDao.update({ id: needInfo.dataValues.id, status: 2 });
      await self.dao.update({ id: solutionInfo.dataValues.id, is_refusal: 1, refusal_notes: actionBody.RefusalContent });
      return system.getResultSuccess();
    })
  }
}
module.exports = NeedSolutionService;